﻿// <copyright file="Program.cs" company="ANUJ Technologies Inc.">
// Copyright (c) 11-13-2011 All Right Reserved
// </copyright>
// <author>Anuj Varma, www.anujvarma.com</author>
// <date>11-13-2011</date>

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using VirtualMode_DataGridView.DomainObjects;

namespace VirtualMode_DataGridView
{
    public partial class VirtualModeForm1 : Form
    {
        BindingList<Customer> _customers = new BindingList<Customer>();
        DateTime startRefreshTime;
        DateTime endRefreshTime;
        const int NUMROWS = 10000;

        public VirtualModeForm1()
        {
            InitializeComponent();
            InitializeGrid();
            InitializeData();
            dataGridView1.Refresh();
        }

        private void InitializeGrid()
        {
            // Enable virtual mode.
            this.dataGridView1.VirtualMode = true;

            // Connect the virtual-mode events to event handlers. 
            this.dataGridView1.CellValueNeeded += new
                DataGridViewCellValueEventHandler(dataGridView1_CellValueNeeded);


            // Add columns to the DataGridView.
            DataGridViewTextBoxColumn companyNameColumn = new
                DataGridViewTextBoxColumn();
            companyNameColumn.HeaderText = "Company Name";
            companyNameColumn.Name = "Company Name";
            DataGridViewTextBoxColumn contactNameColumn = new
                DataGridViewTextBoxColumn();
            contactNameColumn.HeaderText = "Contact Name";
            contactNameColumn.Name = "Contact Name";
            this.dataGridView1.Columns.Add(companyNameColumn);
            this.dataGridView1.Columns.Add(contactNameColumn);
            this.dataGridView1.AutoSizeColumnsMode =
                DataGridViewAutoSizeColumnsMode.AllCells;

            // Set the row count, including the row for new records.
            this.dataGridView1.RowCount = NUMROWS;
        }

        void InitializeData()
        {
            AddCustomerData();
        }

        private void AddCustomerData()
        {
            for (int i = 0; i < NUMROWS; i++)
            {
                string company = "ABC" + i + " Corp";
                string name = "A" + i + "V" + i;
                Customer c = new Customer(company, name);
                this._customers.Add(c);
            }

            // Set the row count, including the row for new records.
            this.dataGridView1.RowCount = NUMROWS;
        }

        private void dataGridView1_CellValueNeeded(object sender, System.Windows.Forms.DataGridViewCellValueEventArgs e)
        {
            // If this is the row for new records, no values are needed.
            if (e.RowIndex == this.dataGridView1.RowCount - 1) return;

            Customer customerTmp = null;

            // Store a reference to the Customer object for the row being painted.     
            customerTmp = (Customer)this._customers[e.RowIndex];

            // Set the cell value to paint using the Customer object retrieved.
            switch (this.dataGridView1.Columns[e.ColumnIndex].Name)
            {
                case "Company Name":
                    e.Value = customerTmp.CompanyName;
                    break;

                case "Contact Name":
                    e.Value = customerTmp.ContactName;
                    break;
            }
        }

    }
}